@@ -29,8 +29,8 @@ const _ = imports.gettext.gettext;
 imports.gi.versions.GtkClutter = '1.0';
 imports.gi.versions.GdPrivate = '1.0';
 imports.gi.versions.Gd = '1.0';
-imports.gi.versions.Tracker = '0.14';
-imports.gi.versions.TrackerMiner = '0.14';
+imports.gi.versions.Tracker = '0.16';
+imports.gi.versions.TrackerMiner = '0.16';
 imports.gi.versions.EvinceDocument = '3.0';
 imports.gi.versions.Goa = '1.0';
 
@@ -99,6 +99,7 @@ const Application = new Lang.Class({
         Gettext.bindtextdomain('gnome-documents', Path.LOCALE_DIR);
         Gettext.textdomain('gnome-documents');
         GLib.set_prgname('gnome-documents');
+        GLib.set_application_name(_("Documents"));
 
         this.parent({ application_id: 'org.gnome.Documents',
                       flags: Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
@@ -398,6 +399,9 @@ const Application = new Lang.Class({
               window_mode: WindowMode.WindowMode.OVERVIEW },
             { name: 'properties',
               callback: this._onActionProperties,
+              window_mode: WindowMode.WindowMode.PREVIEW },
+            { name: 'places',
+              accel: 'F3',
               window_mode: WindowMode.WindowMode.PREVIEW }
         ];
 
