@@ -453,7 +453,7 @@ const Embed = new Lang.Class({
 
     _onFullscreenChanged: function(controller, fullscreen) {
         Gtk.Settings.get_default().gtk_application_prefer_dark_theme = fullscreen;
-        this._toolbar.widget.visible = !fullscreen;
+        this._toolbar.actor.visible = !fullscreen;
         this._toolbar.widget.sensitive = !fullscreen;
     },
 
@@ -483,6 +483,8 @@ const Embed = new Lang.Class({
     },
 
     _onLoadFinished: function(manager, doc, docModel) {
+        docModel.set_sizing_mode(EvView.SizingMode.AUTOMATIC);
+        docModel.set_page_layout(EvView.PageLayout.AUTOMATIC);
         this._toolbar.setModel(docModel);
         this._preview.setModel(docModel);
         this._preview.widget.grab_focus();
@@ -533,7 +535,11 @@ const Embed = new Lang.Class({
     },
 
     getMainToolbar: function() {
-        return this._toolbar;
+        let fullscreen = Application.modeController.getFullscreen();
+        if (fullscreen)
+            return this._preview.getFullscreenToolbar();
+        else
+            return this._toolbar;
     },
 
     getPreview: function() {
