@@ -24,6 +24,7 @@ const EvView = imports.gi.EvinceView;
 const GdPrivate = imports.gi.GdPrivate;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
+const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const GtkClutter = imports.gi.GtkClutter;
 const _ = imports.gettext.gettext;
@@ -35,6 +36,7 @@ const Signals = imports.signals;
 const Application = imports.application;
 const Tweener = imports.util.tweener;
 const MainToolbar = imports.mainToolbar;
+const Places = imports.places;
 const Searchbar = imports.searchbar;
 const Utils = imports.utils;
 const View = imports.view;
@@ -49,10 +51,14 @@ const PreviewView = new Lang.Class({
         this._jobFind = null;
         this._controlsFlipId = 0;
         this._controlsVisible = false;
-        this._selectionChanged = false;
+        this._pageChanged = false;
+        this._viewSelectionChanged = false;
+        this._fsToolbar = null;
+        this._overlayLayout = overlayLayout;
+        this._lastSearch = '';
 
-        Application.modeController.connect('fullscreen-changed',
-            Lang.bind(this, this._onFullscreenChanged));
+        Application.modeController.connect('fullscreen-changed', Lang.bind(this,
+            this._onFullscreenChanged));
 
         this.widget = new Gtk.ScrolledWindow({ hexpand: true,
                                                vexpand: true,
@@ -65,20 +71,10 @@ const PreviewView = new Lang.Class({
 
         this._createView();
 
-        // create thumb bar
-        this._thumbBar = new PreviewThumbnails(this._model);
-        overlayLayout.add(this._thumbBar.actor,
+        // create page nav bar
+        this._navBar = new PreviewNav(this._model);
+        this._overlayLayout.add(this._navBar.actor,
             Clutter.BinAlignment.FILL, Clutter.BinAlignment.END);
-        this._thumbBar.view.connect('selection-changed', Lang.bind(this,
-            function() {
-                this._selectionChanged = true;
-            }));
-
-        // create fullscreen toolbar (hidden by default)
-        this._fsToolbar = new PreviewFullscreenToolbar(this);
-        this._fsToolbar.setModel(this._model);
-        overlayLayout.add(this._fsToolbar.actor,
-            Clutter.BinAlignment.FILL, Clutter.BinAlignment.START);
 
         this.widget.show_all();
 
@@ -117,6 +113,30 @@ const PreviewView = new Lang.Class({
             function() {
                 this._changeRotation(90);
             }));
+        let showPlaces = Application.application.lookup_action('places');
+        showPlaces.connect('activate', Lang.bind(this, this._showPlaces));
+    },
+
+    _showPlaces: function() {
+        let dialog = new Places.PlacesDialog(this._model);
+        dialog.widget.connect('response', Lang.bind(this,
+            function(widget, response) {
+                widget.destroy();
+            }));
+    },
+
+    _onViewSelectionChanged: function() {
+        this._viewSelectionChanged = true;
+        if (!this.view.get_has_selection())
+            this._cancelControlsFlip();
+    },
+
+    _onCanZoomInChanged: function() {
+        this._zoomIn.enabled = this.view.can_zoom_in;
+    },
+
+    _onCanZoomOutChanged: function() {
+        this._zoomOut.enabled = this.view.can_zoom_out;
     },
 
     _createView: function() {
@@ -124,43 +144,52 @@ const PreviewView = new Lang.Class({
         this.widget.add(this.view);
         this.view.show();
 
-        this.view.connect('button-release-event',
-                            Lang.bind(this, this._onButtonReleaseEvent));
-        this.view.connect('key-press-event',
-                            Lang.bind(this, this._onKeyPressEvent));
+        this.view.connect('notify::can-zoom-in', Lang.bind(this,
+            this._onCanZoomInChanged));
+        this.view.connect('notify::can-zoom-out', Lang.bind(this,
+            this._onCanZoomOutChanged));
+        this.view.connect('button-press-event', Lang.bind(this,
+            this._onButtonPressEvent));
+        this.view.connect('button-release-event', Lang.bind(this,
+            this._onButtonReleaseEvent));
+        this.view.connect('key-press-event', Lang.bind(this,
+            this._onKeyPressEvent));
+        this.view.connect('selection-changed', Lang.bind(this,
+            this._onViewSelectionChanged));
     },
 
-    _updateControls: function(visible) {
-        if (this._controlsVisible == visible)
-            return;
-
-        this._controlsVisible = visible;
-
+    _syncControlsVisible: function() {
         if (this._controlsVisible) {
-            if (Application.modeController.getFullscreen())
+            if (this._fsToolbar)
                 this._fsToolbar.show();
-            this._thumbBar.show();
+            this._navBar.show();
         } else {
-            this._fsToolbar.hide();
-            this._thumbBar.hide();
+            if (this._fsToolbar)
+                this._fsToolbar.hide();
+            this._navBar.hide();
         }
     },
 
-    _flipControlsState: function() {
-        this._controlsFlipId = 0;
-        let visible = !this._controlsVisible;
-        this._updateControls(visible);
-
-        return false;
-    },
-
     _onFullscreenChanged: function() {
         let fullscreen = Application.modeController.getFullscreen();
 
-        if (fullscreen && this._controlsVisible)
-            this._fsToolbar.show();
-        else if (!fullscreen)
-            this._fsToolbar.hide();
+        if (fullscreen) {
+            // create fullscreen toolbar (hidden by default)
+            this._fsToolbar = new PreviewFullscreenToolbar(this);
+            this._fsToolbar.setModel(this._model);
+            this._overlayLayout.add(this._fsToolbar.actor,
+                Clutter.BinAlignment.FILL, Clutter.BinAlignment.START);
+
+            this._fsToolbar.connect('show-controls', Lang.bind(this,
+                function() {
+                    this.controlsVisible = true;
+                }));
+        } else {
+            this._fsToolbar.actor.destroy();
+            this._fsToolbar = null;
+        }
+
+        this._syncControlsVisible();
     },
 
     _onKeyPressEvent: function(widget, event) {
@@ -193,6 +222,14 @@ const PreviewView = new Lang.Class({
         return false;
      },
 
+    _flipControlsTimeout: function() {
+        this._controlsFlipId = 0;
+        let visible = this.controlsVisible;
+        this.controlsVisible = !visible;
+
+        return false;
+    },
+
      _cancelControlsFlip: function() {
          if (this._controlsFlipId != 0) {
              Mainloop.source_remove(this._controlsFlipId);
@@ -200,39 +237,48 @@ const PreviewView = new Lang.Class({
          }
      },
 
-     _ensureControlsFlip: function() {
+     _queueControlsFlip: function() {
          if (this._controlsFlipId)
              return;
 
          let settings = Gtk.Settings.get_default();
          let doubleClick = settings.gtk_double_click_time;
 
-         this._controlsFlipId = Mainloop.timeout_add(doubleClick, Lang.bind(this, this._flipControlsState));
+         this._controlsFlipId = Mainloop.timeout_add(doubleClick, Lang.bind(this, this._flipControlsTimeout));
      },
 
+    _onButtonPressEvent: function(widget, event) {
+
+        this._viewSelectionChanged = false;
+
+        return false;
+   },
+
     _onButtonReleaseEvent: function(widget, event) {
         let button = event.get_button()[1];
         let clickCount = event.get_click_count()[1];
 
-        if (button == 1 && clickCount == 1)
-            this._ensureControlsFlip();
+        if (button == 1
+            && clickCount == 1
+            && !this._viewSelectionChanged)
+            this._queueControlsFlip();
         else
             this._cancelControlsFlip();
 
+        this._viewSelectionChanged = false;
+
         return false;
     },
 
     _onScrollbarClick: function() {
-        if (this._controlsVisible)
-            this._flipControlsState();
-
+        this.controlsVisible = false;
         return false;
     },
 
     _onAdjustmentChanged: function() {
-        if (this._controlsVisible && !this._selectionChanged)
-            this._flipControlsState();
-        this._selectionChanged = false;
+        if (!this._pageChanged)
+            this.controlsVisible = false;
+        this._pageChanged = false;
     },
 
     _changeRotation: function(offset) {
@@ -245,7 +291,14 @@ const PreviewView = new Lang.Class({
     },
 
     set controlsVisible(visible) {
-        this._updateControls(visible);
+        // reset any pending timeout, as we're about to change controls state
+        this._cancelControlsFlip();
+
+        if (this._controlsVisible == visible)
+            return;
+
+        this._controlsVisible = visible;
+        this._syncControlsVisible();
     },
 
     startSearch: function(str) {
@@ -263,6 +316,8 @@ const PreviewView = new Lang.Class({
             return;
         }
 
+        this._lastSearch = str;
+
         let evDoc = this._model.get_document();
         this._jobFind = EvView.JobFind.new(evDoc, this._model.get_page(), evDoc.get_n_pages(),
                                            str, false);
@@ -284,8 +339,8 @@ const PreviewView = new Lang.Class({
 
         if (this.view) {
             this.view.destroy();
-            this._thumbBar.hide();
-            this._fsToolbar.hide();
+            this.controlsVisible = false;
+            this._lastSearch = '';
         }
 
         this._model = model;
@@ -293,32 +348,50 @@ const PreviewView = new Lang.Class({
         if (this._model) {
             this._createView();
             this.view.set_model(this._model);
-            this._thumbBar.view.model = model;
-            this._fsToolbar.setModel(model);
+            this._navBar.widget.document_model = model;
+            this._model.connect('page-changed', Lang.bind(this,
+                function() {
+                    this._pageChanged = true;
+                }));
+
         }
     },
 
     getModel: function() {
         return this._model;
+    },
+
+    getFullscreenToolbar: function() {
+        return this._fsToolbar;
+    },
+
+    get lastSearch() {
+        return this._lastSearch;
     }
 });
 Signals.addSignalMethods(PreviewView.prototype);
 
-const PreviewThumbnails = new Lang.Class({
-    Name: 'PreviewThumbnails',
+const PreviewNav = new Lang.Class({
+    Name: 'PreviewNav',
 
     _init: function(model) {
-        this.view = new GdPrivate.SidebarThumbnails({ model: model,
-                                                      visible: true });
-        this.widget = new GdPrivate.ThumbNav({ thumbview: this.view,
-                                               show_buttons: false });
+        this.widget = new GdPrivate.NavBar({ document_model: model });
         this.widget.get_style_context().add_class('osd');
+
+        let button = new Gtk.Button({ action_name: 'app.places',
+                                      child: new Gtk.Image({ icon_name: 'view-list-symbolic',
+                                                             pixel_size: 16 }),
+                                      valign: Gtk.Align.CENTER
+                                    });
+        let button_area = this.widget.get_button_area();
+        button_area.pack_start(button, false, false, 0);
+
         this.actor = new GtkClutter.Actor({ contents: this.widget,
                                             visible: false,
                                             opacity: 0 });
         Utils.alphaGtkWidget(this.actor.get_widget());
 
-        this.widget.show();
+        this.widget.show_all();
     },
 
     show: function() {
@@ -423,7 +496,6 @@ const PreviewSearchbar = new Lang.Class({
     _init: function(previewView) {
         this._previewView = previewView;
         this._previewView.connect('search-changed', Lang.bind(this, this._onSearchChanged));
-        this._lastText = '';
 
         this.parent();
     },
@@ -478,18 +550,16 @@ const PreviewSearchbar = new Lang.Class({
         this.parent();
 
         if (!this._searchEntry.get_text()) {
-            this._searchEntry.set_text(this._lastText);
+            this._searchEntry.set_text(this._previewView.lastSearch);
             this._searchEntry.select_region(0, -1);
         }
 
-        this._lastText = '';
         this._previewView.view.find_set_highlight_search(true);
         this._previewView.startSearch(this._searchEntry.get_text());
     },
 
     hide: function() {
         this._previewView.view.find_set_highlight_search(false);
-        this._lastText = this._searchEntry.get_text();
 
         this.parent();
     }
@@ -502,14 +572,42 @@ const PreviewFullscreenToolbar = new Lang.Class({
     _init: function(previewView) {
         this.parent(previewView);
 
-        this.actor.visible = false;
-        this.widget.sensitive = false;
         this.actor.translation_y = -(this.widget.get_preferred_height()[1]);
+
+        // make controls show when a toolbar action is activated in fullscreen
+        let actionNames = ['gear-menu', 'search'];
+        let signalIds = [];
+
+        actionNames.forEach(Lang.bind(this,
+            function(actionName) {
+                let signalName = 'action-state-changed::' + actionName;
+                let signalId = Application.application.connect(signalName, Lang.bind(this,
+                    function(actionGroup, actionName, value) {
+                        let state = value.get_boolean();
+                        if (state)
+                            this.emit('show-controls');
+                    }));
+
+                signalIds.push(signalId);
+            }));
+
+        this.widget.connect('destroy', Lang.bind(this,
+            function() {
+                signalIds.forEach(
+                    function(signalId) {
+                        Application.application.disconnect(signalId);
+                    });
+            }));
+    },
+
+    handleEvent: function(event) {
+        let res = this.parent(event);
+        if (res)
+            this.emit('search-event-handled');
     },
 
     show: function() {
         this.actor.show();
-        this.widget.sensitive = true;
         Tweener.addTween(this.actor,
                          { translation_y: 0,
                            time: 0.20,
@@ -521,10 +619,12 @@ const PreviewFullscreenToolbar = new Lang.Class({
                          { translation_y: -(this.widget.get_preferred_height()[1]),
                            time: 0.20,
                            transition: 'easeOutQuad',
-                           onComplete: function() {
-                               this.actor.hide();
-                               this.widget.sensitive = false;
-                           },
-                           onCompleteScope: this });
+                           onComplete: Lang.bind(this,
+                               function() {
+                                   this.actor.hide();
+                                   Application.application.change_action_state('search', GLib.Variant.new('b', false));
+                               })
+                         });
     }
 });
+Signals.addSignalMethods(PreviewFullscreenToolbar.prototype);
