@@ -19,9 +19,13 @@
  *
  */
 
+const Gdk = imports.gi.Gdk;
+
 const Lang = imports.lang;
 const Signals = imports.signals;
 
+const Application = imports.application;
+
 const WindowMode = {
     NONE: 0,
     OVERVIEW: 1,
@@ -43,11 +47,23 @@ const ModeController = new Lang.Class({
         if (oldMode == mode)
             return;
 
-        if (mode == WindowMode.OVERVIEW)
-            this.setCanFullscreen(false);
-
+        this.setCanFullscreen(mode == WindowMode.PREVIEW);
         this._mode = mode;
+
         this.emit('window-mode-changed', this._mode, oldMode);
+
+        if (mode != WindowMode.PREVIEW)
+            return;
+
+        let window = Application.application.get_active_window();
+        let gdkWindow = window.get_window();
+
+        if (gdkWindow) {
+            let windowState = gdkWindow.get_state();
+
+            if (windowState & Gdk.WindowState.MAXIMIZED)
+                this.setFullscreen(true);
+        }
     },
 
     getWindowMode: function() {
